;+
; NAME:
;     mc_apotspecflat
;
; PURPOSE:
;     To create the normalize flat field for APO TripleSpec.
;
; CATEGORY:
;     Spectroscopy
;
; CALLING SEQUENCE:
;     mc_apotspecflat,parent,FLATSTAT=flatstat,FLATTHRESH=flatthresh, $
;                     DNORMFLAT=dnormflat,CANCEL=cancel
;
; INPUTS:
;
;
;
; OPTIONAL INPUTS:
;
;
;
; KEYWORD PARAMETERS:
;
;
;
; OUTPUTS:
;
;
;
; OPTIONAL OUTPUTS:
;
;
;
; COMMON BLOCKS:
;
;
;
; SIDE EFFECTS:
;
;
;
; RESTRICTIONS:
;
;
;
; PROCEDURE:
;
;
;
; EXAMPLE:
;
;
;
; MODIFICATION HISTORY:
;
;-


;
;******************************************************************************
;
; ----------------------------Support procedures------------------------------ 
;
;******************************************************************************
;
pro apotspecflat_createflat,files,flatoname,stat,normflat,slitw_pix, $
                            slitw_arc,DARK=darkfile,CANCEL=cancel

  common xspextool_state

  step = cfld(state.w.tracestepsize_fld,3,/EMPTY,CANCEL=cancel)
  if cancel then return

;  Read info from cal file
  
  calfile = cfile(filepath('XD.dat',ROOT=state.r.packagepath,SUB='data'), $
                  WIDGET_ID=state.w.xspextool_base,CANCEL=cancel)
  if cancel then return
  
  mc_readcal,calfile,modename,rotation,norders,orders,slith_pix, $
             slith_pix_range,slith_arc,rppix,step,flatfrac,comwin, $
             edgedeg,norm_nxg,norm_nyg,oversamp,norm_ybuffer,fixed, $
             guesspos,sranges,CANCEL=cancel
  if cancel then return
  

  
  resolvingpower = rppix/slitw_pix
 
  xspextool_message,'Loading the flats...'
  
  call_procedure,state.r.fitsreadprog,files,data,hdrinfo,var, $
                 KEYWORDS=state.r.keywords,ROTATE=rotation,NIMAGES=nimages, $
                 CANCEL=cancel
  if cancel then return

;  Subtract dark if necessary

  if n_elements(DARKFILE) then begin
     
     xspextool_message,'Loading the dark...'

     dark = rotate(readfits(filepath(darkfile,ROOT=state.r.calpath)), $
                            state.r.rotation)
     dark = rebin(reform(dark,state.r.ncols,state.r.nrows,1), $
                  state.r.ncols,state.r.nrows,nimages)

     root    = file_basename(darkfile,'.fits')
     varfile = cfile(filepath(root+'_var.fits',ROOT=state.r.calpath), $
                     WIDGET_ID=state.w.xspextool_base,CANCEL=cancel)
     if cancel then return
     darkvar = rotate(readfits(varfile),state.r.rotation)
     darkvar = rebin(reform(darkvar,state.r.ncols,state.r.nrows,1), $
                     state.r.ncols,state.r.nrows,nimages)
     
     data = temporary(data)-dark
     var  = var+darkvar
     
     darkfile = file_basename(darkfile)
    
  endif

  if nimages ne 1 then begin
     
;  Scale the the images (each set of orders) to the same level.
     
     xspextool_message,'Scaling the flats...'
     
     mc_scaleimgs,data,var,CANCEL=cancel
     if cancel then return
     
;  Combine the images together.
     
     xspextool_message,'Combining the flats...'
     
     if stat le 2 then begin
        
        thresh = cfld(state.w.flatcombthresh_fld,4,/EMPTY,CANCEL=cancel)
        if cancel then return

     endif

     mc_combimgs,data,stat,mean,/UPDATE,THRESH=thresh,IVCUBE=var,CANCEL=cancel
     if cancel then return
     
  endif else mean = data
  if cancel then return

;  Find orders

  ximgtool, mean, SCALING='Hist Eq',ZWIN=0,WID=wid,$
            GROUP_LEADER=state.w.xspextool_base,RANGE='0-Max',BUFFER=1, $
            STDIMAGE=state.r.stdimage,PLOTWINSIZE=state.r.plotwinsize

  xspextool_message,'Locating the orders...'
  
  if state.w.notspex and fixed eq 'Yes' then begin
          
     plots, !x.crange,[guesspos[0],guesspos[0]],color=2
     plots, !x.crange,[guesspos[1],guesspos[1]],color=2
     xranges = sranges
     edgecoeffs = fltarr(2,2,1)
     edgecoeffs[0,*,0] = guesspos
     edgecoeffs[1,*,0] = [0.,0.]
     
  endif else begin

     mc_findorders,mean,guesspos,sranges,step,slith_pix_range,edgedeg,$
                   norm_ybuffer,edgecoeffs,xranges,flatfrac,comwin, $
                   WID=wid,CANCEL=cancel
     if cancel then return
     
  endelse

  ximgtool,mean,SCALING='Hist Eq',GROUP_LEADER=state.w.xspextool_base,$
           RANGE='0-Max',BUFFER=1,ZWIN=0,STDIMAGE=state.r.stdimage, $
           PLOTWINSIZE=state.r.plotwinsize

  if normflat then begin
     
     xspextool_message,'Normalizing the flat...'
     mean = mc_normspecflat(mean,edgecoeffs,xranges,slith_arc,norm_nxg,$
                            norm_nyg,oversamp,norm_ybuffer,RMS=rms,/UPDATE,$
                            WIDGET_ID=state.w.xspextool_base,MODEL=model,$
                            CANCEL=cancel)
     if cancel then return    

;  Replace bad pixels with 
     
     z = where(rotate(*state.d.mask,rotation) eq 0,count)
     if count ne 0 then mean[z] = 1.0
     
  endif else rms = replicate(0.0,norders)
  
;  Write the results to disk.
  
  junk = tag_exist(hdrinfo[*].vals,state.r.irafname,INDEX=idx)
  
  history='This flat was created by 1. scaling '+$
          strjoin(hdrinfo[*].vals.(idx), ', ')+ $
          ' using the median and then 2. '+ $
          state.r.combstats[stat]+ $
          ' combining the resulting images.'
  
  newhdr = hdrinfo[0]
  newhdr.vals.(idx) = flatoname+'.fits'

  mc_writeflat,mc_unrotate(mean,rotation),newhdr,rotation,fix(orders), $
               edgecoeffs,xranges,slith_pix,slith_arc,slitw_pix,slitw_arc, $
               modename,rms,resolvingpower,state.w.version, $
               filepath(flatoname+'.fits',ROOT=state.r.calpath),$
               DARK=darkfile,HISTORY=history,CANCEL=cancel
  if cancel then return
  
  xspextool_message,'Wrote flat to '+filepath(flatoname+'.fits', $
                                              ROOT=state.r.calpath)
  
  if normflat then begin

     ximgtool,state.r.calpath+flatoname+'.fits', ZWIN=0,$
              ZRANGE=[0.9,1.1],GROUP_LEADER=state.w.xspextool_base,$
              BUFFER=1,ROTATION=rotation,STDIMAGE=state.r.stdimage,$
              PLOTWINSIZE=state.r.plotwinsize
     
  endif else begin

     ximgtool,state.r.calpath+flatoname+'.fits', ZWIN=0,$
              /AUTO,GROUP_LEADER=state.w.xspextool_base,$
              BUFFER=1,ROTATION=rotation,STDIMAGE=state.r.stdimage, $
              PLOTWINSIZE=state.r.plotwinsize
     
  endelse

;  Overplot the edges of the orders

  for i = 0,n_elements(orders)-1 do begin

     del = xranges[1,i]-xranges[0,i]+1
     x = findgen(del)+xranges[0,i]
     
     oplot,x,poly(x,edgecoeffs[*,0,i]),COLOR=3
     oplot,x,poly(x,edgecoeffs[*,1,i]),COLOR=3
     
  endfor


end
;
;*****************************************************************************
;
pro apotspecflat_doit,w

  cancel = 0

  common xspextool_state

  ;  Check file readmode.

  index    = (state.r.filereadmode eq 'Index') ? 1:0
  filename = (state.r.filereadmode eq 'Filename') ? 1:0
  
  if index then prefix = cfld(state.w.iprefix_fld,7,/EMPTY,CANCEL=cancel)
  if cancel then return
  
;  Get user inputs.
  
  files = cfld(w.flats_fld,7,/EMPTY,CANCEL=cancel)
  if cancel then return
  files = fsextract(files,INDEX=index,FILENAME=filename,NFILES=nfiles,$
                    CANCEL=cancel)
  if cancel then return
  
  files = mkfullpath(state.r.datapath,files,INDEX=index,FILENAME=filename,$
                     NI=state.r.nint,PREFIX=prefix,SUFFIX='*.fits',$
                     WIDGET_ID=state.w.xspextool_base,/EXIST,CANCEL=cancel)
  if cancel then return
  
     
  dark = strtrim(cfld(w.flatsdark_fld,7,CANCEL=cancel),2)
  if cancel then return
  if strtrim(dark,2) ne '' then $
     tmp = cfile(state.r.calpath+dark,WIDGET_ID=state.w.xspextool_base,$
                 CANCEL=cancel) else delvarx, dark
  if cancel then return
  
  
  flatoname = cfld(w.flatoname_fld,7,/EMPTY,CANCEL=cancel)
  if cancel then return

  slitw_arc = cfld(w.slitw_arc_fld,4,/EMPTY,CANCEL=cancel)
  if cancel then return
  slitw_pix = cfld(w.slitw_pix_fld,4,/EMPTY,CANCEL=cancel)
  if cancel then return
  
  apotspecflat_createflat,files,flatoname,w.combflatstat,w.normflat, $
                          slitw_pix,slitw_arc,DARK=dark,CANCEL=cancel
  
  if keyword_set(BEEP) then beep

end
;
;*****************************************************************************
;
;------------------------------ Event Handler -------------------------------
;
;*****************************************************************************
;
pro apotspecflat_event,event

  common xspextool_state

  widget_control, event.handler, GET_UVALUE = w, /NO_COPY
  widget_control, event.id,  GET_UVALUE = uvalue

  case uvalue of 

;     'Cal File Button': begin
;
;        path= dialog_pickfile(DIALOG_PARENT=state.w.xspextool_base,$
;                              /MUST_EXIST, $
;                              PATH=filepath('',ROOT=state.r.packagepath,$
;                                            SUB='data'),$
;                              FILTER='*.dat',/FIX_FILTER)
;        
;        if path[0] ne '' then $
;           widget_control,state.w.calfile_fld[1],$
;                          SET_VALUE=strjoin(file_basename(path),',',/SINGLE)
;
;     end

     'Combine Flat Field Button': begin
        
        path= dialog_pickfile(DIALOG_PARENT=state.w.xspextool_base,$
                              /MUST_EXIST,PATH=state.r.calpath,$
                              FILTER='*.fits',/FIX_FILTER)
        
        if path ne '' then widget_control,w.combflat_fld[1], $
                                          SET_VALUE = file_basename(path)
        
     end

     'Construct Flat': apotspecflat_doit,w
     
     'Flat Combination Statistic': begin

        w.combflatstat = event.index
        widget_control, w.flatcombthresh_fld[0], $
                        SENSITIVE=(event.index le 2) ? 1:0
        
     end

     'Flat Field Dark Button': begin
        
        path =dialog_pickfile(DIALOG_PARENT=state.w.xspextool_base,$
                              PATH=state.r.calpath,/MUST_EXIST,$
                              FILTER='*.fits',/FIX_FILTER)
        
        if path ne '' then $
           widget_control,w.flatsdark_fld[1],$
                          SET_VALUE =strjoin(file_basename(path),',',/SINGLE)

     end
    
     'Flat Field Images Button': begin
        
        path= dialog_pickfile(DIALOG_PARENT=state.w.xspextool_base,$
                              /MUST_EXIST,PATH=state.r.datapath,$
                              FILTER=['*.fits','*gz'],/FIX_FILTER,$
                              /MULTIPLE_FILES)
        
        if path[0] ne '' then $
           widget_control,w.flats_fld[1],$
                          SET_VALUE=strjoin(file_basename(path),',',/SINGLE)
        
     end

     'Normalize Flat': w.normflat = event.value
     


  endcase

  widget_control, event.handler, SET_UVALUE=w, /NO_COPY

end
;
;*****************************************************************************
;
;------------------------------ Main Program -------------------------------
;
;*****************************************************************************
;
pro mc_apotspecflat,parent,FLATSTAT=flatstat,FLATTHRESH=flatthresh, $
                    DNORMFLAT=dnormflat,CANCEL=cancel

  w = {combstats:['Robust Weighted Mean', $       
                  'Robust Mean (RMS)', $
                  'Robust Mean (Mean Error)', $
                  'Weighted Mean',$
                  'Mean (RMS)', $
                  'Mean (Mean Error)', $
                  'Median (MAD)', $
                  'Median (Median Error)' + $
                  '','Sum'],$
       combflatstat:0,$
       flatcombthresh_fld:[0L,0L],$
       flats_fld:[0L,0L],$
       flatsdark_fld:[0L,0L],$
       flatstat_dl:0L,$
       flatoname_fld:[0L,0L],$
       normflat:keyword_set(DNORMFLAT),$
       normflat_bg:0L,$
       slitw_pix_fld:[0L,0L],$
       slitw_arc_fld:[0L,0L]}

  getfonts,buttonfont,textfont

  row_base = widget_base(parent,$
                         /ROW)
  
     col1_base = widget_base(row_base,$
                             /COLUMN,$
                             /FRAME)
     
;           row = widget_base(col1_base,$
;                             /ROW,$
;                             /BASE_ALIGN_CENTER)
;           
;              button = widget_button(row,$
;                                     FONT=buttonfont,$
;                                     VALUE='Cal File',$
;                                     UVALUE='Cal File Button')
;              
;              fld = coyote_field2(row,$
;                                  LABELFONT=buttonfont,$
;                                  FIELDFONT=textfont,$
;                                  TITLE=':',$
;                                  UVALUE='Cal File Field',$
;                                  XSIZE=15,$
;                                  TEXTID=textid) 
;              w.calfile_fld = [fld,textid]

           row = widget_base(col1_base,$
                             /ROW,$
                             /BASE_ALIGN_CENTER)
           
              fld = coyote_field2(row,$
                                  LABELFONT=buttonfont,$
                                  FIELDFONT=textfont,$
                                  TITLE='Slit Width (Pix/Arc):',$
                                  UVALUE='Slit Width Pix',$
                                  XSIZE=5,$
                                  TEXTID=textid) 
              w.slitw_pix_fld = [fld,textid]
              
              fld = coyote_field2(row,$
                                  LABELFONT=buttonfont,$
                                  FIELDFONT=textfont,$
                                  TITLE='/ ',$
                                  UVALUE='Slit Width Arc',$
                                  XSIZE=5,$
                                  TEXTID=textid) 
              w.slitw_arc_fld = [fld,textid]
              
        row = widget_base(col1_base,$
                          /ROW,$
                          /BASE_ALIGN_CENTER)
        
           button = widget_button(row,$
                                  FONT=buttonfont,$
                                  VALUE='Raw Flat Images',$
                                  UVALUE='Flat Field Images Button')
           
           fld = coyote_field2(row,$
                               LABELFONT=buttonfont,$
                               FIELDFONT=textfont,$
                               TITLE=':',$
                               UVALUE='Flat Field Images Field',$
                               XSIZE=15,$
                               TEXTID=textid) 
           w.flats_fld = [fld,textid]
           
           row = widget_base(col1_base,$
                             /ROW,$
                             /BASE_ALIGN_CENTER)
           
              button = widget_button(row,$
                                     FONT=buttonfont,$
                                     VALUE='Full Dark Name',$
                                     UVALUE='Flat Field Dark Button')
              
              fld = coyote_field2(row,$
                                  LABELFONT=buttonfont,$
                                  FIELDFONT=textfont,$
                                  TITLE=':',$
                                  UVALUE='Flat Field Dark Field',$
                                  XSIZE=15,$
                                  TEXTID=textid) 
              w.flatsdark_fld = [fld,textid]
                     
                  
        row = widget_base(col1_base,$
                          /ROW,$
                          /BASE_ALIGN_CENTER)
        
           w.flatstat_dl = widget_droplist(row,$
                                           VALUE=w.combstats,$
                                           FONT=buttonfont,$
                                           TITLE='Statistic:',$
                                           UVALUE='Flat Combination Statistic')

           if keyword_set(FLATSTAT) then begin
              
              w.combflatstat = where(w.combstats eq flatstat)
              widget_control, w.flatstat_dl, $
                              SET_DROPLIST_SELECT=w.combflatstat
              
           endif
           
           fld = coyote_field2(row,$
                               LABELFONT=buttonfont,$
                               FIELDFONT=textfont,$
                               TITLE='Thresh:',$
                               UVALUE='Robust Threshold',$
                               XSIZE=5,$
                               VALUE=flatthresh,$
                               TEXTID=textid)
           w.flatcombthresh_fld = [fld,textid]
           
        w.normflat_bg = cw_bgroup(col1_base,$
                                  FONT=buttonfont,$
                                  ['No','Yes'],$
                                  /ROW,$
                                  /RETURN_INDEX,$
                                  /NO_RELEASE,$
                                  /EXCLUSIVE,$
                                  LABEL_LEFT='Normalize Flat:',$
                                  SET_VALUE=w.normflat,$
                                  UVALUE='Normalize Flat')
        
        fld = coyote_field2(col1_base,$
                            LABELFONT=buttonfont,$
                            FIELDFONT=textfont,$
                            TITLE='Output Flat Name:',$
                            UVALUE='Output Flat Name',$
                            XSIZE=15,$
                            TEXTID=textid) 
        w.flatoname_fld = [fld,textid]
        
        button = widget_button(col1_base,$
                               FONT=buttonfont,$
                               VALUE='Construct Flat',$
                               UVALUE='Construct Flat')
        
; Start the Event Loop. This will be a non-blocking program.

  XManager, 'xmc_apotspecflat', $
            row_base, $
            EVENT_HANDLER='apotspecflat_event',$
            /NO_BLOCK

; Put state variable into the user value of the top level base.

  widget_control, row_base, SET_UVALUE=w, /NO_COPY


end
